      program linsim
c
c     this program performs similation of a self-exciting point process
c  which is also stimulated by another given point process data; non
c  stationary poisson trend is also included in the intensity
c  function.
c
c     structure of the program
c
c          linsim
c             i---input
c             i---initl
c             i---simxx----comfac
c             i     i------duf
c             i     i------unifor
c             i     +------fx--------ptrend
c             +---output
c
c     this program is designed by y. ogata, and programmed by y. ogata
c  and k. katsura, inst. statist. math., tokyo. (31/01/85)
c
c     references
c
c  y. ogata (1981). "on lewis' simulation method for point processes."
c     ieee information theory, vol. it-27, pp. 23-31.
c  y. ogata & h. akaike (1982). "on linear intensity models for mixed
c     doubly stochastic poisson and self-exciting point processes." j.
c     royal statist. soc. b, vol. 44, pp. 102-107.
c  y. ogata, h. akaike and k. katsura (1982). "the application of linear
c     intensity models to the investigatio of causal relations between a
c     point process and another stochastic process."  ann. inst.
c     statist math., vol. 34. pp. 373-387.
c  y. ogata and k. katsura (1985). "point process model with linearly
c     parametrized intensity for the application to earthquake data."
c     essays in time series and allied processes (festscrift for prof.
c     e. j. hannan), j. gani and m. b. priestley eds., j.appl. probab.
c     vol. 23a, to appear.
c
      implicit real*8(a-h,o-z)
      dimension xx(10000),yy(10000)
      dimension axx(100),axy(100),axz(100)
      dimension ei(100),ej(100),fi(100),fj(100),lf(51,51)
c
      call input(kxx,kxy,kxz,t,c,d,axx,axy,axz,yy,mm,ptxmax,kmax)
c
      call initl(kxx,axx,c,fxxmax)
      call initl(kxy,axy,d,fxymax)
c
      call simxx(kxx,kxy,kxz,t,c,d,axx,axy,axz,fxxmax,fxymax,
     &           xx,yy,ei,ej,fi,fj,kmax,ptxmax,lf,i1,j1)
c
      call output(xx,yy,i1,j1,t)
c
      stop
      end
      subroutine input(kxx,kxy,kxz,t,c,d,axx,axy,axz,yy,mm,ptxmax,
     &                 kmax)
c
c  inputs
c     kxx: kxx-1 is the order of laguerre polynomial (lgp) of the
c          transfer function (= response function) from the data events
c          xx to xx ( trf; xx --> xx ).
c     kxy: kxy-1 is the oeder of lgp trf; yy --> xx
c     kxz: kxz-1 is the order of polynomial for xx data
c     t:   length of time interval in which events take place
c     c,d: exponential coefficients in lgp corresponding to
c          xx and xy, respectively
c     axx(i),axy(i),axz(i):
c        coefficients of the corresponding polynomials
c     ptxmax:
c          an upper bound of trend polynomial
c     mm:  number of the input data yy(i)
c     yy(i):
c          input point process data
c
      implicit real*8(a-h,o-z)
c     dimension axx(1),axy(1),axz(1),yy(1),fmt(10)
      dimension axx(1),axy(1),axz(1),yy(1)
      character*8 fmt(10)
      open(4,file='linsim.init')
      read(4,*) kxx,kxy,kxz
      read(4,*) t,c,d
      if(kxx.ne.0) read(4,*) (axx(i),i=1,kxx)
      if(kxy.ne.0) read(4,*) (axy(i),i=1,kxy)
      if(kxz.ne.0) read(4,*) (axz(i),i=1,kxz)
      read(4,*) ptxmax
      read(4,*) in
c     read(4,11) fmt
      open(in,file='linsim.data')
      read(in,*) mm
c     read(in,fmt) (yy(i),i=1,mm)
      read(in,*) (yy(i),i=1,mm)
      yy(mm+1)=t
      write(6,3) kxx,kxy,kxz
      write(6,4) t,c,d
      if(kxx.ne.0) write(6,5) (axx(i),i=1,kxx)
      if(kxy.ne.0) write(6,6) (axy(i),i=1,kxy)
      if(kxz.ne.0) write(6,7) (axz(i),i=1,kxz)
      write(6,8) ptxmax
      write(6,12) in
c     write(6,13) fmt
      write(6,9) mm
      write(6,10) (yy(i),i=1,mm)
      kmax=max(kxx,kxy,kxz)
    1 format(8i10)
    2 format(8f10.3)
    3 format(//1h ,' kxx =',i5,3x,' kxy =',i5,3x,'kxz =',i5)
    4 format(//1h ,' t =',f12.5,3x,' c =',d13.5,3x,' d =',d13.5)
    5 format(//1h ,' axx(i),i=1,kxx'/(1h ,5d13.5))
    6 format(//1h ,' axy(i),i=1,kxy'/(1h ,5d13.5))
    7 format(//1h ,' axz(i),i=1,kxz'/(1h ,5d13.5))
    8 format(//1h ,' ptxmax =',d13.5)
    9 format(//1h ,' mm =',i5)
   10 format(//1h ,' input data (yy(i),i=1,mm)'/(1h ,5f12.5))
   11 format(10a8)
   12 format(//1h ,' in =',i5)
   13 format(//1h ,' fmt = ',10a8)
      return
      end
      subroutine initl(kx,ax,c,fmax)
c
c     calculation of an upper bound of a response function
c
      implicit real*8 (a-h,o-z)
      dimension ax(1)
      fmax=0.0
      if(kx.eq.0) go to 120
      do 100 i=1,1000
      x=(1000-i)*6.0d0/c/1000
      fx=ax(1)
      if(kx.eq.1) go to 130
      do 110 k=2,kx
  110 fx=fx+ax(k)*x**(k-1)
  130 continue
      fx=fx*exp(-c*x)
      if(fmax.lt.fx) fmax=fx
  100 continue
  120 continue
      fmax=fmax*1.5
      return
      end
      subroutine simxx(kxx,kxy,kxz,t,c,d,axx,axy,axz,fxxmax,fxymax,
     &                 xx,yy,ei,ej,fi,fj,kmax,ptxmax,lf,i1,j1)
      implicit real*8(a-h,o-z)
      dimension axx(1),axy(1),axz(1),xx(1),yy(1),ei(1),ej(1),fi(1),fj(1)
      dimension lf(51,51)
      real*4r
      do 100 k=1,kmax
      ei(k)=0.0
      ej(k)=0.0
      fi(k)=0.0
      fj(k)=0.0
  100 continue
c-------------------------
      call comfac(lf)
c-------------------------
      x=0.0
      xx(1)=0.0
      uity=ptxmax
      i=0
      j=0
      ij=0
   30 continue
c----------------------------------------------------------------
      call duf(i,j,x,duity,xx,yy,axx,axy,kxx,kxy,c,d,ei,ej,fi,fj,
     &         ptxmax)
c----------------------------------------------------------------
c
      if(duity.lt.uity) uity=duity
   20 continue
c--------------------
  130 call unifor(r)
c--------------------
   40 continue
      e=-alog(r)/uity
      x=x+e
      if(x.gt.t) go to 80
    6 format(2i3,7f10.5)
      if(kxy.eq.0) go to 110
      if(x.le.yy(j+1)) go to 110
      x=yy(j+1)
c-----------------------------------------------------------------
      call fx(i,j,yy(j+1),dmx,axx,axy,axz,kxx,kxy,kxz,c,d,lf,ei,ej,
     &        fi,fj,xx,yy)
c-----------------------------------------------------------------
      do 120 k=1,kxy
  120 ej(k)=fj(k)
      j=j+1
      uity=uity+fxymax
      go to 130
c
  110 continue
c
c------------------------------------------------------------
      call fx(i,j,x,xity,axx,axy,axz,kxx,kxy,kxz,c,d,lf,ei,ej,
     &        fi,fj,xx,yy)
c------------------------------------------------------------
      probx=xity/uity
      prob=xity/uity
      if(prob.le.1.0d00)go to 50
      write(6,1) prob
      stop
   50 continue
   60 continue
c-----------------------
      call unifor(r)
c-----------------------
      ij=ij+1
      if(probx.le.r)go to 30
      i=i+1
      xx(i)=x
      do 10 k=1,kxx
      ei(k)=fi(k)
   10 continue
      uity=uity+fxxmax
      go to 20
   80 continue
      i1=i
      j1=j
    1 format(1h ,'warning: is ptmax correct?  prob=',f10.5)
      return
      end
      subroutine comfac(lf)
c
c     calculation of the combination factorials
c     n & m should be interpreted to be n-1 & m-1, respectively.
c
      dimension lf(51,51)
      lf(1,1)=1
      lf(2,1)=1
      lf(2,2)=1
      do 10 n=3,51
      lf(n,1)=1
      lf(n,n)=1
      do 20 m=2,n-1
      lf(n,m)=lf(n-1,m-1)+lf(n-1,m)
   20 continue
   10 continue
      return
      end
      subroutine duf(i,j,x,duity,xx,yy,axx,axy,kxx,kxy,c,d,ei,ej,fi,
     &               fj,ptxmax)
c
c     decreasing process except jumps, which is always greater than
c     the intensity process in subroutine fx.
c
      implicit real * 8 (a-h,o-z)
      dimension bxx(100),bxy(100)
      dimension axx(1),axy(1),ei(1),ej(1),fi(1),fj(1)
      dimension xx(1),yy(1)
      ixf=1
      iyf=1
      cxp=0.0
      cyp=0.0
      do 30 ix=1,kxx
      bxx(ix)=max(axx(ix),0.d0)
      cx=bxx(ix)/(c/2)**(ix-1) *ixf
      cxp=max(cx,cxp)
      ixf=ixf*ix
   30 continue
      do 50 iy=1,kxy
      bxy(iy)=max(axy(iy),0.d0)
      cy=bxy(iy)/(d/2)**(iy-1) *iyf
      cyp=max(cy,cyp)
      iyf=iyf*iy
   50 continue
      if(i.eq.0) go to 10
      dxxi=x-xx(i)
      ecdxxi=0.0d00
      if(-c/2*dxxi.ge.-20.0) ecdxxi=exp(-c/2*dxxi)
      fi(1)=ecdxxi*(ei(1)+1.d0)
   10 continue
      if(j.eq.0) go to 20
      dyyj=x-yy(j)
      ecdyyj=0.0d00
      if(-d/2*dyyj.ge.-20.0) ecdyyj=exp(-d/2*dyyj)
      fj(1)=ecdyyj*(ej(1)+1.d0)
   20 continue
      xity=ptxmax+cxp*fi(1)+cyp*fj(1)
      duity=xity
      return
      end
      subroutine unifor(r)
c     generation of pseudo-random numbers
      data ir/584287/
      ir=ir*48828125
      if(ir) 10,20,20
   10 ir=(ir+2147483647)+1
   20 r=float(ir)*0.4656613e-9
      return
      end
      subroutine fx(i,j,x,xity,axx,axy,axz,kxx,kxy,kxz,c,d,lf,ei,
     &              ej,fi,fj,xx,yy)
c
c     intensity processes
c
      implicit real * 8 (a-h,o-z)
      dimension axx(1),axy(1),axz(1),ei(1),ej(1),fi(1),fj(1),lf(51,51)
      dimension xx(1),yy(1)
      if(i.eq.0) go to 30
      dxxi=x-xx(i)
      ecdxxi=0.0d00
      if(-c*dxxi.ge.-20.0) ecdxxi=exp(-c*dxxi)
      do 70 ix=1,kxx
      ff=ei(1)*lf(ix,1)+1.d0
      if(ix.eq.1) go to 85
      do 80 jx=2,ix
      ff=ff*dxxi+ei(jx)*lf(ix,jx)
   80 continue
   85 continue
      fi(ix)=ecdxxi*ff
   70 continue
   30 continue
      if(j.eq.0) go to 40
      dyyj=x-yy(j)
      ecdyyj=0.0d00
      if(-d*dyyj.ge.-20.0) ecdyyj=exp(-d*dyyj)
      do 90 ix=1,kxy
      ff=ej(1)*lf(ix,1)+1.d0
      if(ix.eq.1) go to 105
      do 100 jx=2,ix
      ff=ff*dyyj+ej(jx)*lf(ix,jx)
  100 continue
  105 continue
      fj(ix)=ecdyyj*ff
   90 continue
   40 continue
      call ptrend(x,ptx,axz,kxz)
      xity=ptx
      if(i.eq.1) go to 15
      do 10 k=1,kxx
      xity=xity+axx(k)*fi(k)
   10 continue
   15 continue
      if(j.eq.1) go to 25
      do 20 k=1,kxy
      xity=xity+axy(k)*fj(k)
   20 continue
   25 continue
      return
      end
      subroutine ptrend(x,ptx,axz,kxz)
c
c     trend polynomial; it is possible to includes here another
c     deterministic functions for an intensity of non-stationary
c     poisson process such as a fourier series for cyclic component.
c
      implicit real*8(a-h,o-z)
      dimension axz(1)
      ptx=0.0
      do 10 i=1,kxz
      ptx=ptx+axz(i)*x**(i-1)
   10 continue
      return
      end
      subroutine output(xx,yy,i1,j1,t)
      implicit real*8(a-h,o-z)
      dimension xx(1),yy(1)
c     simulation data print out
      write(6,3)t,i1,j1
      write(6,1) (xx(k),k=1,i1)
      write(6,2) (yy(k),k=1,j1)
c     simulation data punch out
c     write(7,5)t,i1,j1
c     write(7,4)(xx(k),k=1,i1)
c     write(7,4)(yy(k),k=1,j1)
    1 format(//1h ,'simulated data (x(k),k=1,nn)'/(1h ,5f12.5))
    2 format(//1h ,'input data (y(k),k=1,mm)'/(1h ,5f12.5))
    3 format(1h //1h ,' t =',f12.5,3x,' nn =',i5,3x,' mm =',i5)
    4 format(5f10.4)
    5 format(f10.4,2i10)
      return
      end
